# -*- coding: utf-8 -*-

from kaio import Options
from functools import partial

opts = Options()

get = partial(opts.get, section='Debug')

class DebugMixin(object):
    """Securty base settings"""

    DEBUG_TOOLBAR_CONFIG = {'INTERCEPT_REDIRECTS': False}
    INTERNAL_IPS = ('127.0.0.1', )

    @property
    def DEBUG(self):
        return get('DEBUG', False)

    @property
    def TEMPLATE_DEBUG(self):
        return get('TEMPLATE_DEBUG', self.DEBUG)

    @property
    def ENABLE_DEBUG_TOOLBAR(self):
        enabled = get('ENABLE_DEBUG_TOOLBAR', self.DEBUG)
        if enabled:
            try:
                import debug_toolbar
            except ImportError:
                return False
            else:
                self.add_to_installed_apps()
                self.add_to_middleware_classes()

        return enabled

    @property
    def ALLOWED_HOSTS(self):
        hosts = get('ALLOWED_HOSTS', 'www.change-me.net')
        return [h.strip() for h in hosts.split(',')]

    def add_to_installed_apps(self):
        if 'debug_toolbar' not in self.INSTALLED_APPS:
            self.INSTALLED_APPS += ('debug_toolbar', )

    def add_to_middleware_classes(self):
        if 'debug_toolbar.middleware.DebugToolbarMiddleware' not in self.MIDDLEWARE_CLASSES:
            self.MIDDLEWARE_CLASSES += (
                'debug_toolbar.middleware.DebugToolbarMiddleware',
            )



# -*- coding: utf-8 -*-

from kaio import Options
from os.path import abspath, join
from functools import partial

opts = Options()
get = partial(opts.get, section='Paths')

class PathsMixin(object):

    @property
    def APP_ROOT(self):
        return get('APP_ROOT', abspath('.'))

    @property
    def MEDIA_ROOT(self):
        return get('MEDIA_ROOT', abspath(join(self.APP_ROOT, 'media')))

    @property
    def STATIC_URL(self):
        return get('STATIC_URL', '/static/')

    @property
    def MEDIA_URL(self):
        return get('MEDIA_URL', '/media/')

    @property
    def STATIC_ROOT(self):
        return get(
            'STATIC_ROOT',
            abspath(join("/tmp", "%s-static" % self.APP_SLUG)))



# -*- coding: utf-8 -*-

import os
from kaio import Options
from functools import partial

opts = Options()
get = partial(opts.get, section='Email')

class EmailMixin(object):
    """Settings para enviar emails"""

    # Django settings: https://docs.djangoproject.com/en/1.6/ref/settings/#email-backend

    @property
    def EMAIL_BACKEND(self):
        return get('EMAIL_BACKEND', 'django.core.mail.backends.smtp.EmailBackend')

    @property
    def EMAIL_FILE_PATH(self):
        return get('EMAIL_FILE_PATH', None)

    @property
    def EMAIL_HOST(self):
        return get('EMAIL_HOST', 'localhost')

    @property
    def EMAIL_HOST_PASSWORD(self):
        return get('EMAIL_HOST_PASSWORD', '')

    @property
    def EMAIL_HOST_USER(self):
        return get('EMAIL_HOST_USER', '')

    @property
    def EMAIL_PORT(self):
        return get('EMAIL_PORT', 25)

    @property
    def EMAIL_SUBJECT_PREFIX(self):
        return get('EMAIL_SUBJECT_PREFIX', '[Django] ')

    @property
    def EMAIL_USE_TLS(self):
        return get('EMAIL_USE_TLS', False)


    # django-yubin settings: http://django-yubin.readthedocs.org/en/latest/settings.html

    @property
    def MAILER_PAUSE_SEND(self):
        return get('MAILER_PAUSE_SEND', False)

    @property
    def MAILER_USE_BACKEND(self):
        return get('MAILER_USE_BACKEND', 'django.core.mail.backends.smtp.EmailBackend')

    @property
    def MAILER_MAIL_ADMINS_PRIORITY(self):
        try:
            from django_yubin import constants
            priority = constants.PRIORITY_HIGH
        except Exception:
            priority = 1
        return get('MAILER_MAIL_ADMINS_PRIORITY', priority)

    @property
    def MAILER_MAIL_MANAGERS_PRIORITY(self):
        return get('MAILER_MAIL_MANAGERS_PRIORITY', None)

    @property
    def MAILER_EMPTY_QUEUE_SLEEP(self):
        return get('MAILER_EMPTY_QUEUE_SLEEP', 30)

    @property
    def MAILER_LOCK_WAIT_TIMEOUT(self):
        return get('MAILER_LOCK_WAIT_TIMEOUT', -1)

    @property
    def MAILER_LOCK_PATH(self):
        return get("MAILER_LOCK_PATH", os.path.join(self.APP_ROOT, "send_mail"))



# -*- coding: utf-8 -*-

from kaio import Options
from functools import partial

opts = Options()
get = partial(opts.get, section='Database')

class DatabasesMixin(object):

    def DATABASES(self):
        engine = get('DATABASE_ENGINE', 'sqlite3')
        if 'django.db.backends' in engine:
            ENGINE = engine
        else:
            ENGINE = 'django.db.backends.' + engine

        return {
            'default': {
                'ENGINE': ENGINE,
                'NAME': get('DATABASE_NAME', 'db.sqlite'),
                'USER': get('DATABASE_USER', None),
                'PASSWORD': get('DATABASE_PASSWORD', ''),
                'HOST': get('DATABASE_HOST', ''),
                'PORT': get('DATABASE_PORT', ''),
                'OPTIONS': {},
            }
        }



# -*- coding: utf-8 -*-

from kaio import Options
from functools import partial

opts = Options()
get = partial(opts.get, section='Logs')

class LogsMixin(object):
    """Django Logging configuration"""

    @property
    def LOG_LEVEL(self):
        return get('LOG_LEVEL', 'DEBUG').upper()

    @property
    def DJANGO_LOG_LEVEL(self):
        return get('DJANGO_LOG_LEVEL', 'ERROR').upper()

    @property
    def LOG_FILE(self):
        return get('LOG_FILE', '')

    @property
    def EXTRA_LOGGING(self):
        """
        lista modulos con los distintos niveles a logear y su
        nivel de debug

        Por ejemplo:

            [Logs]
            EXTRA_LOGGING = oscar.paypal:DEBUG, django.db:INFO

        """

        input_text = get('EXTRA_LOGGING', '')
        modules = input_text.split(',')
        if input_text:
            modules = input_text.split(',')
            modules = [x.split(':') for x in modules]
        else:
            modules = []
        return modules

    @property
    def SENTRY_ENABLED(self):
        enabled = get('SENTRY_ENABLED', False)
        try:
            import raven
        except ImportError:
            return False

        if enabled:
            self.add_sentry_to_installed_apps()
        return enabled

    @property
    def SENTRY_DSN(self):
        return get('SENTRY_DSN', '')

    # The best way to propagate logs up to the root logger is to prevent
    # Django logging configuration and handle it ourselves.
    #
    # http://stackoverflow.com/questions/20282521/django-request-logger-not-propagated-to-root/22336174#22336174
    # https://docs.djangoproject.com/en/1.9/topics/logging/#disabling-logging-configuration
    LOGGING_CONFIG = None

    @property
    def LOGGING(self):
        config = {
            'version': 1,
            'disable_existing_loggers': True,
            'formatters': self.formatters,
            'filters': self.filters,
            'handlers': self.handlers,
            'loggers': self.loggers,
        }
        import logging.config
        logging.config.dictConfig(config)
        return config

    @property
    def handlers(self):
        handlers = {}

        handlers['default'] = {
            'level': self.LOG_LEVEL,
            'class': 'logging.StreamHandler',
            'formatter': 'apsldefault'
        }

        if self.LOG_FILE:
            handlers['default']['class'] = 'logging.FileHandler'
            handlers['default']['filename'] = self.LOG_FILE

        handlers['mail_admins'] = {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }

        if self.SENTRY_ENABLED:
            handlers['sentry'] = {
                'level': 'ERROR',
                'class': 'raven.contrib.django.raven_compat.handlers.SentryHandler',
            }

        return handlers

    @property
    def loggers(self):
        loggers = {}

        loggers[''] = {
            'handlers': ['default'],
            'level': self.LOG_LEVEL,
            'propagate': True,
        }

        loggers['rq.worker'] = {
            'handlers': ['default'],
            'level': self.LOG_LEVEL,
            'propagate': False,
        }

        loggers['requests.packages.urllib3'] = {
            'handlers': ['default'],
            'level': self.LOG_LEVEL,
            'propagate': False,
        }

        loggers['django'] = {
            'handlers': ['default'],
            'level': self.DJANGO_LOG_LEVEL,
            'propagate': False,
        }

        if self.EXTRA_LOGGING:
            try:
                for module, level in self.EXTRA_LOGGING:
                    loggers[module] = {
                        'handlers': ['default'],
                        'level': level,
                        'propagate': False,
                    }
            except Exception as exc:
                import sys
                sys.stderr.write(exc)

        if self.SENTRY_ENABLED:
            loggers['']['handlers'].append('sentry')
            loggers['django']['handlers'].append('sentry')
            loggers['raven'] = {
                'handlers': ['default'],
                'level': self.LOG_LEVEL,
                'propagate': False,
            }
            loggers['sentry.errors'] = {
                'handlers': ['default'],
                'level': self.LOG_LEVEL,
                'propagate': False,
            }
        return loggers

    @property
    def formatters(self):
        return {
            'apsldefault': {
                'format': "[%(asctime)s] %(levelname)s %(name)s-%(lineno)s %(message)s"
            }
        }

    @property
    def filters(self):
        return {
            'require_debug_false': {
                '()': 'django.utils.log.RequireDebugFalse',
            }
        }

    def add_sentry_to_installed_apps(self):
        if 'raven.contrib.django.raven_compat' not in self.INSTALLED_APPS:
            self.INSTALLED_APPS += ('raven.contrib.django.raven_compat', )



# -*- coding: utf-8 -*-

from kaio import Options

opts = Options()


def get(value, default):
    return opts.get(value, default, section='Security')


class SecurityMixin(object):
    """Securty base settings"""

    @property
    def SECRET_KEY(self):
        return get('SECRET_KEY', u'sysadmin, canvia la secret key!!!!')

    @property
    def ALLOWED_HOSTS(self):
        return [h.strip() for h in \
                get('ALLOWED_HOSTS', '*').split(',')]




# -*- coding: utf-8 -*-
# @author: bcabezas@apsl.net

import sys

from clint.textui import puts, colored
from django.core.management.base import NoArgsCommand

from kaio import Options


def module_to_dict(module, omittable=lambda k: k.startswith('_')):
    """
    Converts a module namespace to a Python dictionary. Used by get_settings_diff.
    """
    return dict([(k, repr(v)) for k, v in module.__dict__.items() if not omittable(k)])


class Command(NoArgsCommand):
    help = """Displays differences between the current settings.py and Django's
    default settings. Settings that don't appear in the defaults are
    followed by "###"."""

    requires_model_validation = False

    def handle_noargs(self, **options):
        # Inspired by Postfix's "postconf -n".
        from django.conf import settings

        # Because settings are imported lazily, we need to explicitly load them.
        settings._setup()

        user_settings = module_to_dict(settings._wrapped)

        opts = Options()
        pformat = "%-25s = %s"
        puts('')
        for section in opts.sections:
            puts(colored.green("[%s]" % section))
            for key, kaio_value in opts.items(section):
                keycolor = colored.magenta(key)
                if key in user_settings:
                    keycolor = colored.blue(key)

                default_value = opts.options[key].default_value
                value = kaio_value or default_value

                if sys.version_info[0] < 3:
                    value = unicode(value).encode('utf8')
                else:
                    value = str(value)

                try:
                    puts(pformat % (keycolor, value))
                except Exception as e:
                    raise e
            puts('')



# -*- coding: utf-8 -*-
# bcabezas@apsl.net

from django.core.management.base import NoArgsCommand

from kaio import Options


def module_to_dict(module, omittable=lambda k: k.startswith('_')):
    "Converts a module namespace to a Python dictionary. Used by get_settings_diff."
    return dict([(k, repr(v)) for k, v in module.__dict__.items() if not omittable(k)])


def clint_encode(value):
    """clint puts crashess when value is unicode. so we always encode"""
    return value


class Command(NoArgsCommand):
    help = """Displays differences between the current settings.py and Django's
    default settings. Settings that don't appear in the defaults are
    followed by "###"."""

    requires_model_validation = False

    def handle_noargs(self, **options):
        # Inspired by Postfix's "postconf -n".
        from django.conf import settings, global_settings

        # Because settings are imported lazily, we need to explicitly load them.
        settings._setup()

        user_settings = module_to_dict(settings._wrapped)
        default_settings = module_to_dict(global_settings)

        opts = Options()
        from clint.textui import puts, colored
        pformat = "%30s: %-30s %-30s %-30s"
        puts('')
        puts(pformat % (
            colored.white('Option'),
            colored.cyan('APP Value'),
            colored.cyan('INI Value'),
            colored.green('APP Default')))
        puts('')
        for section in opts.sections:
            puts(pformat % (colored.green("[%s]" % section), '', '', ''))
            for key, kaio_value in opts.items(section):
                keycolor = colored.magenta(key)
                if key in user_settings:
                    value = colored.green(user_settings[key])
                    keycolor = colored.blue(key)
                else:
                    value = colored.green(opts.options[key].get_value_or_default())

                default_value = opts.options[key].default_value
                kaio_value = kaio_value if kaio_value else repr(kaio_value)

                puts(pformat % (
                    keycolor,
                    clint_encode(value),
                    colored.white(clint_encode(kaio_value)),
                    clint_encode(default_value)))

            puts('')

        puts(colored.white("No configurables directamente en INI (estáticos o compuestos por otros):"))
        puts()

        not_configured = set(user_settings.keys()) - set(opts.keys())
        #not_configured = not_configured - set([
            #'INSTALLED_APPS',
            #'MIDDLEWARE_CLASSES',
            #'CONTEXT_PROCESSORS',
            #])
        pformat = "%30s: %-50s"
        puts(pformat % (
            colored.white('Option'),
            colored.cyan('Value')))
        for key in sorted(not_configured):
            if key not in default_settings:
                puts(pformat % (colored.blue(key),
                    user_settings[key]))
                    #'###'))
            elif user_settings[key] != default_settings[key]:
                puts(pformat % (
                    colored.blue(key),
                    colored.green(user_settings[key])))
                    #colored.white(default_settings[key])))




# -*- coding: utf-8 -*-

from os.path import join, abspath
from kaio import Options
from configurations import Settings
from functools import partial

opts = Options()
get = partial(opts.get, section='Filer')

class FilerMixin(object):
    """Settings para django-filer y easy_thumbnails"""

    THUMBNAIL_PROCESSORS = (
        'easy_thumbnails.processors.colorspace',
        'easy_thumbnails.processors.autocrop',
        # 'easy_thumbnails.processors.scale_and_crop',
        'filer.thumbnail_processors.scale_and_crop_with_subject_location',
        'easy_thumbnails.processors.filters',
    )

    @property
    def FILER_IS_PUBLIC_DEFAULT(self):
        return get('FILER_IS_PUBLIC_DEFAULT', True)

    @property
    def FILER_ENABLE_PERMISSIONS(self):
        return get('FILER_ENABLE_PERMISSIONS', False)

    @property
    def FILER_DEBUG(self):
        return get('FILER_DEBUG', False)

    @property
    def FILER_ENABLE_LOGGING(self):
        return get('FILER_ENABLE_LOGGING', False)

    @property
    def FILER_0_8_COMPATIBILITY_MODE(self):
        get('FILER_0_8_COMPATIBILITY_MODE', False)

    @property
    def THUMBNAIL_DEBUG(self):
        return get('THUBMNAIL_DEBUG', False)

    @property
    def THUMBNAIL_QUALITY(self):
        return get('THUMBNAIL_QUALITY', 85)

    @property
    def FILER_CUSTOM_NGINX_SERVER(self):
        """If true will serve secure file trough XNginxXAccelRedirectServer"""
        return get('FILER_CUSTOM_NGINX_SERVER', False)

    @property
    def default_file_storage(self):
        """Common storage for filer configs"""
        return getattr(
            Settings, 'DEFAULT_FILE_STORAGE',
            'django.core.files.storage.FileSystemStorage')

    @property
    def FILER_CUSTOM_SECURE_MEDIA_ROOT(self):
        """Secure media root
        As in filer settings, defaults to MEDIA_ROOT/../smedia"""
        return opts.get(
            'FILER_CUSTOM_SECURE_MEDIA_ROOT',
            abspath(join(self.MEDIA_ROOT, '..', 'smedia')))

    @property
    def filer_private_files_path(self):
        return abspath(
            join(
                self.FILER_CUSTOM_SECURE_MEDIA_ROOT,
                'filer_private'
            ))

    @property
    def filer_private_thumbnails_path(self):
        return abspath(
            join(
                self.FILER_CUSTOM_SECURE_MEDIA_ROOT,
                'filer_private_thumbnails'))

    @property
    def FILER_SERVERS(self):
        """Filer config to be served from XNginxXAccelRedirectServer
        see http://django-filer.readthedocs.org/en/0.9.4/secure_downloads.html#secure-downloads
        """
        if not self.FILER_CUSTOM_NGINX_SERVER:
            return {}
        else:
            return {
                'private': {
                    'main': {
                        'ENGINE': 'filer.server.backends.nginx.NginxXAccelRedirectServer',
                        'OPTIONS': {
                            'location': self.filer_private_files_path,
                            'nginx_location': '/nginx_filer_private',
                        },
                    },
                    'thumbnails': {
                        'ENGINE': 'filer.server.backends.nginx.NginxXAccelRedirectServer',
                        'OPTIONS': {
                            'location': self.filer_private_thumbnails_path,
                            'nginx_location': '/nginx_filer_private_thumbnails',
                        },
                    },
                },
            }

    @property
    def FILER_STORAGES(self):
        """Filer config to set custom private media path
        http://django-filer.readthedocs.org/en/0.9.4/settings.html#filer-storages
        """
        if not self.FILER_CUSTOM_NGINX_SERVER:
            return {}

        return {
            'public': {
                'main': {
                    'ENGINE': self.default_file_storage,
                    'OPTIONS': {},
                    'UPLOAD_TO': 'filer.utils.generate_filename.by_date',
                    'UPLOAD_TO_PREFIX': 'filer_public',
                },
                'thumbnails': {
                    'ENGINE': self.default_file_storage,
                    'OPTIONS': {},
                    'THUMBNAIL_OPTIONS': {
                        'base_dir': 'filer_public_thumbnails',
                    },
                },
            },
            'private': {
                'main': {
                    'ENGINE': 'filer.storage.PrivateFileSystemStorage',
                    'OPTIONS': {
                        'location': self.filer_private_files_path,
                        'base_url': '/smedia/filer_private/',
                    },
                    'UPLOAD_TO': 'filer.utils.generate_filename.by_date',
                    'UPLOAD_TO_PREFIX': '',
                },
                'thumbnails': {
                    'ENGINE': 'filer.storage.PrivateFileSystemStorage',
                    'OPTIONS': {
                        'location': self.filer_private_thumbnails_path,
                        'base_url': '/smedia/filer_private_thumbnails/',
                    },
                    'THUMBNAIL_OPTIONS': {},
                },
            },
        }
